// Page.h
//
// page class

#include <time.h>

class PageList;

class Page
{
public:
	Page(const char* title, int flags = 0);
	~Page();

	const char*	GetTitle() const		{ return mTitle; }
	int			GetFlags() const		{ return mFlags; }
	int			GetSeqNo() const		{ return mSeqNo; }
	time_t		GetTime() const			{ return mTime; }
	bool		IsLoaded() const		{ return !!mASCII; }
	const char*	GetData() const			{ return mASCII; }
	const char*	GetSpacedTitle() const	{ return mSpacedTitle; }

	const char* GetDateString() const;
	const char* GetTimeString() const;
	const char*	GetFilename() const;

	int			ModifyFlags(int on, int off);

	void		ScanForLinks();

private:

	friend class PageSet;

	char*		mTitle;						// title of page
	char*		mSpacedTitle;				// spaced title of page
	char*		mASCII;						// page contents in ASCII
	Page*		mNext;						// next page in radix
	int			mFlags;						// flags
	int			mSeqNo;						// sequence number
	time_t		mTime;						// filetime
	void		(PageSet::*mCB)(Page* pp);	// post-change callback

	char**		mLink;						// list of links
	int			mLinkCnt;					// number of links
};

enum PageFlags
{
	PF_KEEP = 1,		// page is kept in RAM
	PF_CHANGED = 2,		// page differs from file version
	PF_META = 4,		// page is a meta-page
	PF_PAGEPARM = 8,	// page link should include ?page=<current page name>
	PF_ADMINEDIT = 16,	// page can only be editing by an administrator
	PF_ORPHANED = 32,	// temp flag for checking orphans
	PF_NONWIKI = 64,	// text page
};

enum CharFmt;
