// PageFormatting.cpp
//
// formatting stuff for Page class

#include "Inc.h"

// WriteAsHTML
//
// write out a page as HTML

bool PageSet::WriteAsHTML(Page* page, FILE* outfd, const UserInfo* user, bool convert) const
{
	// load data and make headings
	LoadPage(page);
	
	const char*	title = page->GetTitle();
	const char* stitle = page->GetSpacedTitle();

	char		heading[1024];
	if (!convert)	sprintf(heading, "<a href=\"ReverseLink?page=%s\">%s</a>", title, stitle);
	else			sprintf(heading, "%s", stitle);

	// write from HTML template
	const char*	lp = mPageTemplate->mASCII;
	bool		footer = false;
	char*       incpagetitle = NULL;

	while (WriteFromTemplate(lp, outfd, stitle, heading, footer, user, &incpagetitle, convert))
	{
		if (footer)
		{
			// write footer
			WriteFooter(page, outfd, user);
		}
		else
		{
			if (incpagetitle)
			{
				// Write the included Wikipage
				WriteIncludedPage(page, incpagetitle, outfd, user, convert);
				delete[] incpagetitle;
				incpagetitle= NULL;
			}
			else
			{
				// write page
				OutputHTML(page, page->mASCII, outfd, user, convert);
			}
		}
	}

	UnloadPage(page);

	return true;
}

// OutputHTML
//
// write HTML for a Wiki string

void PageSet::OutputHTML(const Page* page, const char* ls, FILE* outfd, const UserInfo* user, bool convert) const
{
	WikiParser	parser;

	parser.WikiToHTML(outfd, ls, Plain, page, user, convert);
}

// WriteFooter
//
// write the footer

void PageSet::WriteFooter(const Page* page, FILE* outfd, const UserInfo* user) const
{
	if (page)
	{
//		if (!user->Auth('w') || ((page->mFlags & PF_ADMINEDIT) && !user->Auth('a')))
//		{
//			OutputHTML(page, mWikiSystemFooter->mASCII, outfd, user);
//		}
//		else
		{
			if (page != mWikiFooter)
			{
				OutputHTML(page, mWikiFooter->mASCII, outfd, user);
			}
			else
			{
				OutputHTML(page, STD_FOOTER, outfd, user);
			}
		}
	}
	else
	{
		OutputHTML(page, mWikiSystemFooter->mASCII, outfd, user);
	}
}

// WriteIncludedPage
//
// writes a Wikipage that was include in the pagetemplate

void PageSet::WriteIncludedPage(const Page* page, const char* incpagetitle, FILE* outfd, const UserInfo* user, bool convert) const
{
	// Write the included page
	Page*	pp = cfg.Pages->FindPage(incpagetitle);

	if (!pp)
	{
		fprintf(outfd, "%s<a href=\"%s\">?</a>", incpagetitle, incpagetitle);
	}
	else if (pp->IsLoaded() || (pp->GetFlags() & PF_META) || !cfg.Pages->LoadPage(pp))
	{
		// loaded already (cannot have recursion) or meta - don't include
		fprintf(outfd, "(Can't include %s)<br>", incpagetitle);
	}
	else
	{
		OutputHTML(page, pp->mASCII, outfd, user, convert);
		UnloadPage(pp);
	}
}

// OutputPageLink
//
// output a page link

void PageSet::OutputPageLink(const Page* page, const char* name, FILE* fd, bool convert) const
{
	Page*	pp = FindPage(name);
	char*	spaced = MakeSpacedTitle(name);

	char*	enc = StringDup(name);

//	UTFEncode(enc);
	URLEncode(enc);

	if (pp)
	{
		// link
		if (pp->mFlags & PF_PAGEPARM)
		{
			if (!page || (page->mFlags & PF_META) || convert)
			{
				fprintf(fd, "%s", spaced);
			}
			else
			{				
				fprintf(fd, "<a href=\"%s?page=%s\">%s</a>", enc, page->mTitle, spaced);
			}
		}
		else
		{
			if (!convert)	fprintf(fd, "<a href=\"%s\">%s</a>", enc, spaced);
			else			fprintf(fd, "<a href=\"%s.html\">%s</a>", enc, spaced);
		}
	}
	else
	{
		// dangling link
		if (!convert)	fprintf(fd, "%s<a href=\"%s\">?</a>", spaced, enc);
		else			fprintf(fd, "%s", spaced);
	}

	delete[] spaced;
	delete[] enc;
}
