// WikiServer.h
//
// Wiki server class

#include "Config.h"
#include "WikiUser.h"
#include "Page.h"
#include "PageSet.h"
#include "Defaults.h"
#include "HTag.h"
#include "WikiParser.h"

class WikiServer : public Server
{
public:
	WikiServer(int port, FILE* logfd, const char* rootdir);
	~WikiServer();

protected:
	const char*	GetServerName();
	int			Get(TempFile& file, HTTP_RQ* rq, Headers& hdrs);

private:
	PageSet*	mPageSet;
	Page*		mEditPage;
	Page*		mDumpAllPages;
	Page*		mListAllPages;
	Page*		mListOrphans;
	Page*		mRecentChanges;
	Page*		mReverseLink;
	Page*		mAdminLogin;
	Page*		mAuthorLogin;

	int ServeWikiName(const char* uri, TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int GetRaw(TempFile& file, const char* filename, const char* ext, Headers& hdrs, const UserInfo* user);
	int	Cmd(Page* page, TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);

	int	EditPage(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int	DumpAllPages(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int	ListAllPages(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int ListOrphans(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int RecentChanges(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int ReverseLink(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int AdminLogin(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int AuthorLogin(TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);

	int SubmitPage(const char* pagename, TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int TitleSearch(const char* pagename, TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
	int FullSearch(const char* pagename, TempFile& file, HTTP_RQ* rq, Headers& hdrs, const UserInfo* user);
};
