// WikiUser
//
// Wiki user handling

#include "Inc.h"

// LoadUsers
//
// load users into config

void WikiConfig::LoadUsers(const char* file)
{
	UserList.DeleteAll();
	delete[] UserTable;

	UserList.AddFromFileData(file);

	if (!UserList.Count())
	{
		// add the default user
		UserList.AddHeader(":", "rwa");
	}

	// create user table
	int	ii;

	UserCount = UserList.Count();
	UserTable = new UserInfo[UserCount];

	for (ii = 0; ii < UserCount; ii++)
	{
		const Header*	hdr = UserList.Index(ii);

		UserTable[ii].SetInfo(hdr->mName, hdr->mData);
		UserTable[ii].SetDefaultHomepage(cfg.DefaultPage);
	}
}

// QueryAuth
//
// find username:password key in the userlist, and return its value, else a default

const UserInfo* WikiConfig::QueryAuth(const char* auth) const
{
	int	ix = UserList.FindIndex(auth);
	if (ix == -1)
	{
		ix = UserList.FindIndex(":");
		if (ix == -1)	return 0;
	}

	return &UserTable[ix];
}

// UserInfo
//
// constructor & destructor

UserInfo::UserInfo()
{
	username = 0;
	password = 0;
	perms = 0;
	homepage = 0;
}

UserInfo::~UserInfo()
{
	delete[] username;
	delete[] password;
	delete[] perms;
	delete[] homepage;
}

void UserInfo::SetInfo(const char* _key, const char* _value)
{
	delete[] username;
	delete[] password;
	delete[] perms;
	delete[] homepage;

	username = 0;
	password = 0;
	perms = 0;
	homepage = 0;

	char*	pwd = StringDup(_key);

	char*	uname;
	pwd = StringSplit(pwd, ':', uname);

	if (!uname)		return;	// leave in uninitialized state (will never be used)

	if (!uname[0])
	{
		delete[] uname;
		uname = 0;
	}

	username = uname;
	password = pwd;

	char*	hpage = StringDup(_value);

	char*	prm;
	hpage = StringSplit(hpage, ':', prm);

	if (!prm || !prm[0])
	{
		// no : so it's just permissions
		perms = hpage;
	}
	else
	{
		perms = prm;
		if (hpage[0])	homepage = hpage;
	}
}

void UserInfo::SetDefaultHomepage(const char* hpage)
{
	if (!homepage)	homepage = StringDup(hpage);
}

bool UserInfo::Auth(char action) const
{
	if (perms && strchr(perms, action))	return true;

	return false;
}