// headers.h
//
// header set

struct Header
{
	char*		mName;		// header name
	char*		mData;		// header data
	Header*		mNext;		// next header
};

class Headers
{
public:
	Headers();
	~Headers();

	// add a header
	void	AddHeader(const char* name, const char* data);

	// delete all headers
	void	DeleteAll();

	// add headers from an HTTP request
	// (must not include first line)
	// the HTTP request *must* be properly terminated ([<CR>]<LF>[<CR>]<LF>)
	void	AddFromHTTP(const char* httprq);

	// add headers from a CGI string
	void	AddFromCGI(const char* cgistr);

	// add headers from a cookie string
	void	AddFromCookie(const char* cookie);

	// find header
	const Header*	Find(const char* name, int which = 0) const;

	// get data else default
	const char*		Lookup(const char* name, const char* def = 0) const;

	// array-like access
	const Header*	Index(int which) const;

	// find index
	int				FindIndex(const char* name, int which = 0) const;

	// count headers
	int		Count() const;

	// write out
	void	Write(FILE* fd) const;

	// INI-type files
	bool	AddFromFile(const char* file);
	bool	AddFromFileData(const char* filedata);
	bool	WriteToFile(const char* file) const;

private:
	Header*	mParms;
};
