// main.cpp
//
// main function for Wiki server

#include <stdio.h>
#include <stdlib.h>

#include "server.h"

#include "Wiki/WikiServer.h"

#include "os.h"

static int		port = -1;			// server port
static bool		clearlog = false;	// clear log
static char*	logfile = NULL;		// log filename
static char*	pageroot= "Pages";	// folder from which the pages will be served

static void parse_cmdline(int argc, char** argv);

int main(int argc, char** argv)
{
	// init OS
	OS_Init();

	// get command-line parameters
	parse_cmdline(argc, argv);

	// handle defaults
	if (port == -1)	port = 80;

	FILE*	logfd;

	if (logfile)
	{
		logfd = fopen(logfile, clearlog ? "w" : "a");
		if (!logfd)
		{
			fprintf(stderr, "Can't open logfile %s\n", logfile);
			exit(1);
		}
		OS_DetachConsole();
	}
	else
	{
		logfd = stdout;
	}

	// create server object
	Server*	server = new WikiServer(port, logfd, pageroot);

	// run server
	int rc = server->Run() ? 0 : 1;

	return rc;
}

static void parse_cmdline(int argc, char** argv)
{
	// go through arguments
	argc--;
	argv++;

	while (argc)
	{
		char*	arg = argv[0];

		if (arg[0] == '-')
		{
			char	opt = arg[1];

			arg += 2;

			if ((opt == 'p') || (opt == 'l') || opt == 'r')
			{
				// expects an argument
				if (!arg[0])
				{
					// use next one
					argc--;
					argv++;
					if (!argc)	break;	// no more
					arg = argv[0];
				}
			}

			if (opt == 'p')
			{
				// -p<n> = set port
				if (port != -1)
				{
					fprintf(stderr, "Port already set to %d\n", port);
					exit(1);
				}
				port = atoi(arg);
				if ((port < 0) || (port > 65535))
				{
					fprintf(stderr, "Port %d is not valid\n", port);
					exit(1);
				}
			}
			else if (opt == 'l')
			{
				// -l<name> = set log filename
				if (logfile != NULL)
				{
					fprintf(stderr, "Log file already set to %s\n", logfile);
					exit(1);
				}
				logfile = arg;
			}
			else if (opt == 'r')
			{
				// -r<path> = set the path from which the files will be served
#ifdef WIN32
			    DWORD dwAttributes;

				// check if directory exists
				// 0xffffffff means that the call failed
				dwAttributes = GetFileAttributes(arg); 

		        if ( (dwAttributes == 0xffffffff) || !(dwAttributes & FILE_ATTRIBUTE_DIRECTORY)) 
				{
					fprintf(stderr, "%s is not a valid directory\n", arg);
					exit(1);
				}
#endif
				pageroot = arg;
			}
			else if (opt == 'c')
			{
				// -c = clear log file
				clearlog = true;
			}
			else break;	// bad option
		}
		else
		{
			break;
		}

		argc--;
		argv++;
	}

	if (argc)
	{
		fprintf(stderr, "Syntax:\n\nWikiServer [-p<port>] [-l<logfile>] [-r<path>] [-c]\n\n");
		fprintf(stderr, "<port> = port number (0 - 65535)\n");
		fprintf(stderr, "<logfile> = log filename (defaults to stdout)\n");
		fprintf(stderr, "<path> = directory from which the files should be served (defaults to Pages)\n");
		fprintf(stderr, "-c = clear logfile first\n");
		exit(1);
	}
}
