// os.h
//
// OS-specific stuff

#include <time.h>

// OS_Init
//
// do any OS-specific initialization

void OS_Init();

// OS_DetachConsole
//
// detach from console window if possible

void OS_DetachConsole();

// OS_InitSocketAPI
//
// initialize socket API

bool OS_InitSocketAPI();

// OS_TermSocketAPI
//
// terminate socket API

void OS_TermSocketAPI();

// OS_GetSocketError
//
// get socket error code

int OS_GetSocketError();

// OS_MakeTempFilename
//
// make a temp filename using the given tag - the filename
// is in the system temp directory and its name is uniquely
// determined by the tag and the current thread id - the name
// must always be the same if called in the same thread with
// identical parameters

void OS_MakeTempFilename(char* buffer, const char* tag);

// OS_Yield
//
// yield (sleep) for a short time (e.g. 10ms)

void OS_Yield();

// OS_ScanDir
//
// given a directory, calls the callback for every file
// in the directory
//
// in callback: dirname = directory name e.g. C:\Pages
//              filename = filename in directory e.g. LikeThis.wik
//              basename = base filename e.g. LikeThis
//              ext = extension e.g. wik

typedef void (*OS_ScanDirCB)(const char* dirname, const char* filename, const char* basename, const char* ext, void* apikey);

void OS_ScanDir(const char* dirname, OS_ScanDirCB cb, void* apikey);

// OS_GetModifiedTime
//
// get the modified time for a file

time_t OS_GetModifiedTime(const char* pathname);

// OS_SetModifiedTime
//
// set the modified time for a file

void OS_SetModifiedTime(const char* pathname, time_t time);

// OS_GetDateString
//
// convert a time_t to a (static) date string

const char* OS_GetDateString(time_t time);

// OS_GetTimeString
//
// convert a time_t to a (static) time string

const char* OS_GetTimeString(time_t time);

// OS_IsSameDay
//
// see if two time_ts are on the same day

bool OS_IsSameDay(time_t t1, time_t t2);

//
// POSIX autoconf stuff
//

#ifdef POSIX
#include "config.h"
#endif

//
// sanos config stuff
//

#ifdef SANOS
#define _MAX_PATH 256
#define MAX_PATH 256
typedef unsigned long u_long;
#define _mkdir(D) mkdir(D, 0666)
#endif
