// os_sanos
//
// sanos OS-specific routines

#ifdef SANOS

#include <os.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "os.h"
#include "utility.h"

// OS_Init
//
// do any OS-specific initialization

void OS_Init()
{
  fmode = O_TEXT;
}

// OS_DetachConsole
//
// detach from console window if possible

void OS_DetachConsole()
{
}

// OS_InitSocketAPI
//
// initialize socket API

bool OS_InitSocketAPI()
{
  return true;
}

// OS_TermSocketAPI
//
// terminate socket API

void OS_TermSocketAPI()
{
}

// OS_GetSocketError
//
// get socket error code

int OS_GetSocketError()
{
  return errno;
}

// OS_MakeTempFilename
//
// make a temp filename using the given tag

void OS_MakeTempFilename(char* buffer, const char* tag)
{
  char *tmp = getenv("tmp");
  sprintf(buffer, "%s/%s-%d.txt", tmp, tag, gettid());
}

// OS_Yield
//
// yield for a short time

void OS_Yield()
{
  sleep(0);
}

// OS_GetModifiedTime
//
// get the modified time for a file

time_t OS_GetModifiedTime(const char *pathname)
{
  struct stat sb;

  stat(pathname, &sb);
  return sb.st_mtime;
}

// OS_SetModifiedTime
//
// set the modified time for a file

void OS_SetModifiedTime(const char* pathname, time_t time)
{
  struct utimbuf tbuf;

  tbuf.ctime = -1;
  tbuf.actime = -1;
  tbuf.modtime = time;

  utime(pathname, &tbuf);
}

// OS_ScanDir
//
// given a directory, calls the callback for every file
// in the directory

void OS_ScanDir(const char *dirname, void (*cb)(const char *dirname, const char *filename, const char *basename, const char *ext, void *apikey), void *apikey)
{
  int dir;
  struct direntry dirp;
  dir = _opendir(dirname);
  if (dir < 0) return;

  while (_readdir(dir, &dirp, 1) > 0)
  {
    struct stat sb;
    char path[MAXPATH];

    strcpy(path, dirname);
    strcat(path, "/");
    strcat(path, dirp.name);

    if (stat(path, &sb) < 0) continue;
    if ((sb.st_mode & S_IFMT) == S_IFDIR) continue;

	// parse the filename
	char basename[MAXPATH];
	char ext[MAXPATH];

	char *s = dirp.name;
	char *e = s + strlen(s);

	while (e > s)
	{
	  if (e[-1] == '.') break;
	  e--;
	}

	if (e > s)
	{
	  e--;
	  memcpy(basename, s, e - s);
	  basename[e - s] = 0;
	  strcpy(ext, e + 1);
	}
	else
	{
	  strcpy(basename, s);
	  ext[0] = 0;
	}

	// callback
	cb(dirname, s, basename, ext, apikey);
  }
  close(dir);
}

// GetTimeSys
//
// get system time for time_t

static void GetTimeSys(struct tm *stime, time_t t)
{
  struct tm *rc = localtime(&t);

  if (rc) 
	memcpy(stime, rc, sizeof(struct tm));
  else
	memset(stime, 0, sizeof(struct tm));
}

// OS_GetDateString
//
// convert a time_t to a (static) date string

const char *OS_GetDateString(time_t t)
{
  static char date[64];
  struct tm stime;

  gmtime_r(&t, &stime); 
  sprintf(date, "%s %d, %d", MonthName(stime.tm_mon + 1), stime.tm_mday, stime.tm_year + 1900);
  return date;
}

// OS_GetTimeString
//
// convert a time_t to a (static) time string

const char *OS_GetTimeString(time_t t)
{
  static char time[128];
  struct tm stime;

  gmtime_r(&t, &stime); 
  sprintf(time, "%02d:%02d", stime.tm_hour, stime.tm_min);
  return time;
}

// OS_IsSameDay
//
// see if two time_ts are on the same day

bool OS_IsSameDay(time_t t1, time_t t2)
{
  struct tm	st1, st2;

  gmtime_r(&t1, &st1);
  gmtime_r(&t2, &st2);

  if (st1.tm_year != st2.tm_year) return false;
  if (st1.tm_mon != st2.tm_mon) return false;
  if (st1.tm_mday != st2.tm_mday) return false;

  return true;
}

#endif	// sanos
