// server.h
//
// abstract server class

#include "os_sockets.h"

#include "httprq.h"
#include "tempfile.h"

class Server
{
public:
	Server(int port, FILE* logfd);
	virtual ~Server();

	bool		Run();

protected:
	virtual const char*	GetServerName() = 0;

	void		Log(char* fmt, ...);
	void		FatalError(char* fmt, ...);

	void		SetCookie(Headers& hdrs, const char* key, const char* value) const;

	void		ShutdownAfterTransaction()		{ mAlive = false; }

	virtual int	Get(TempFile& file, HTTP_RQ* rq, Headers& hdrs)		{ return 500; }

private:
	int				mPort;
	FILE*			mLog;
	bool			mAlive;

	void			CreateErrorPages();
	void			CreateErrorPage(int status, char* desc, char* verbiage);

	void			AddServerHeaders(Headers& hdrs);
	int				GetPort(SOCKET& s);
	void			HTTPError(SOCKET s, int status);
	HTTP_RQ*		ReadMessage(SOCKET s);
	void			RunCmd(SOCKET s, HTTP_RQ* rq);
};
