// tempfile.cpp
//
// temporary file

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "utility.h"
#include "tempfile.h"
#include "os.h"

TempFile::TempFile(const char* tag)
{
	mTag = StringDup(tag);
	mFD = NULL;
	mCloneFile = NULL;
	mCleanUp = false;
}

TempFile::~TempFile()
{
	Delete();
	delete[] mCloneFile;
}

void TempFile::Delete()
{
	Close();

	// delete temp file if required
	if (mCleanUp)
	{
		char	fn[_MAX_PATH];
		GetName(fn);
		remove(fn);
		mCleanUp = false;
	}
}

void TempFile::CloneExistingFile(const char* filename)
{
	Delete();

	delete[] mCloneFile;
	mCloneFile = StringDup(filename);
}

FILE* TempFile::Open(const char* perm)
{
	Close();

	if (mCloneFile)
	{
		if (strchr(perm, 'r'))
		{
			mFD = fopen(mCloneFile, perm);
		}
	}
	else
	{
		char	fn[_MAX_PATH];
		GetName(fn);

		mFD = fopen(fn, perm);

		if (mFD && strchr(perm, 'w'))
		{
			mCleanUp = true;
		}
	}
	
	return mFD;
}

void TempFile::Close()
{
	if (mFD)
	{
		fclose(mFD);
		mFD = NULL;
	}
}

void TempFile::GetName(char* fn)
{
	OS_MakeTempFilename(fn, mTag);
}
