// utility.h
//
// utilities

#define WIKISERVER_VERSION "1.1.m2"

// CS_Compare
//
// compare strings in charset

int CS_Compare(const char* s1, const char* s2, bool ignore_case);

// CS_islower
// CS_isupper
// CS_isalpha
// CS_isalnum
// CS_isdigit
//
// char attributes

bool CS_islower(char c);
bool CS_isupper(char c);
bool CS_isalpha(char c);
bool CS_isalnum(char c);
bool CS_isdigit(char c);

// StringSplit
//
// split text at character (at)
// frees text and returns new text buffer, plus new
// part buffer - user is responsible for freeing
// these buffers, and for not passing in static
// data!

char* StringSplit(char* text, char at, char*& part);


// StripSpaces
//
// Strip spaces from str

void StripSpaces(char* str);


// StringDup
//
// duplicate string

char* StringDup(const char* str);
char* StringDup(const char* start, const char* end);

// URLEncode
// URLDecode
//
// encode/decode data from URL mapping
//
// functions delete input string and return new output buffer

void URLEncode(char*& data);
void URLDecode(char*& data);

// UTFDecode
//
// decode from UTF-8 into an 8-bit character set
//
// conversion is in-place (it can only shrink the string)

void UTFDecode(char* data);

// UTFEncode
//
// encode from ISO-8859-1 to UTF-8
//
// function deletes input string and returns new output buffer

void UTFEncode(char*& data);

// GetX
//
// decode an item from a string; return newly-allocated buffer
// and advance ls past the item's end

char* GetLine(const char*& ls);
char* GetAlphaNumeric(const char*& ls);
char* GetURL(const char*& ls);
char* GetNumber(const char*& ls);
char* GetISBN(const char*& ls, char rawisbn[11]);

// MonthName
//
// 1 = January, ..., 12 = December

const char* MonthName(int ix);

// MatchWildcardString
//
// see if a wildcard string is within another string

bool MatchWildcardString(const char* string, const char* search, bool recursing = false);

// DecodeBase64
//
// decode a Base64 string [see RFC 1341]

char* DecodeBase64(const char* input, int& len);

// GetSysTime
//
// get the current time in system format as a (static) string

const char* GetSysTime();

// GetSysDate
//
// get the current date in system format as a (static) string

const char* GetSysDate();

// LoadFile
//
// load a file into a new buffer

char* LoadFile(const char* filename);


// Capitalise
//
// Makes each word start with a capital

void Capitalise(char* str);


// IsValidTitle
//
// is the string a valid title?

bool IsValidTitle(const char* tptr);

// IsAcceptableTitle
//
// is the string an acceptable title?

bool IsAcceptableTitle(const char* tptr);

// MakeSpacedTitle
//
// make a wiki name into a spaced wiki name

char* MakeSpacedTitle(const char* title);
